%feature("docstring") OT::SaltelliSensitivityAlgorithm
"Sensitivity analysis using Saltelli method.

Available constructors:
    SaltelliSensitivityAlgorithm(*inputDesign, outputDesign, N*)

    SaltelliSensitivityAlgorithm(*distribution, N, model, computeSecondOrder*)

    SaltelliSensitivityAlgorithm(*experiment, model, computeSecondOrder*)

Parameters
----------
inputDesign : :class:`~openturns.Sample`
    Design for the evaluation of sensitivity indices, obtained
    thanks to the SobolIndicesAlgorithmImplementation.Generate method
outputDesign : :class:`~openturns.Sample`
    Design for the evaluation of sensitivity indices, obtained as the evaluation
    of a Function (model) on the previous inputDesign
distribution : :class:`~openturns.Distribution`
    Input probabilistic model.
    Should have independent copula
experiment : :class:`~openturns.WeightedExperiment`
    Experiment for the generation of two independent samples.
N : int
    Size of samples to generate
computeSecondOrder : bool
    If True, design that will be generated contains elements for the evaluation
    of second order indices.

Notes
-----
This class is concerned with analyzing the influence the random vector
:math:`\\vect{X} = \\left( X^1, \\ldots, X^{n_X} \\right)` has on a random variable
:math:`Y^k` which is being studied for uncertainty, by using the [saltelli2002]_
method for the evaluation of both first and total order indices.

These last ones are respectively given as follows:

.. math::

   \\begin{array}{ccc}
   \\hat{V_i} & = & \\frac{1}{n}\\sum_{k=1}^{n} G(B_k)  G(E_k) - (\\frac{1}{n} \\sum_{k=1}^{n} G(A_k)) (\\frac{1}{n} \\sum_{k=1}^{n} G(B_k)) \\\\
   \\hat{V_{-i}} & = & \\frac{1}{n}\\sum_{k=1}^{n} G(A_k)  G(E_k) - G_0^2
   \\end{array}

The variance of the estimator is computed using:

.. math::

    \\begin{array}{ccc}
    U_i = \\left(\\textsuperscript{c}Y_i^B \\textsuperscript{c}Y_i^E, \\left(\\textsuperscript{c}Y_i^{A}\\right)^2 \\right)^T \\\\
    \\Psi_i(x, y) = \\frac{x}{y}
    \\end{array}

.. math::

    \\begin{array}{ccc}
    U_{-i} = \\left(\\textsuperscript{c}Y_i^A \\textsuperscript{c}Y_i^E, \\left(\\textsuperscript{c}Y_i^{A}\\right)^2 \\right)^T \\\\
    \\Psi_{-i}(x, y) = 1 - \\frac{x}{y}
    \\end{array}

See also
--------
SobolIndicesAlgorithm

Examples
--------
>>> import openturns as ot
>>> ot.RandomGenerator.SetSeed(0)
>>> formula = ['sin(pi_*X1)+7*sin(pi_*X2)^2+0.1*(pi_*X3)^4*sin(pi_*X1)']
>>> model = ot.SymbolicFunction(['X1', 'X2', 'X3'], formula)
>>> distribution = ot.ComposedDistribution([ot.Uniform(-1.0, 1.0)] * 3)
>>> # Define designs to pre-compute
>>> size = 100
>>> inputDesign = ot.SobolIndicesExperiment(distribution, size).generate()
>>> outputDesign = model(inputDesign)
>>> # sensitivity analysis algorithm
>>> sensitivityAnalysis = ot.SaltelliSensitivityAlgorithm(inputDesign, outputDesign, size)
>>> print(sensitivityAnalysis.getFirstOrderIndices())
[0.249038,0.425106,-0.0623136]"

